package com.mockitotutorial.happyhotel.booking;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.time.LocalDate;

import static org.mockito.Mockito.*;

class Test08Spies {

	private BookingService bookingService;
	private PaymentService paymentServiceMock;
	private RoomService roomServiceMock;
	private BookingDAO bookingDAOMock;
	private MailSender mailSenderMock;

	@BeforeEach
	void setup() {
		paymentServiceMock = mock(PaymentService.class);
		roomServiceMock = mock(RoomService.class);
		bookingDAOMock = spy(BookingDAO.class);
		mailSenderMock = mock(MailSender.class);

		bookingService = new BookingService(paymentServiceMock, roomServiceMock, bookingDAOMock, mailSenderMock);
	}

	@Test
	void should_MakeBooking_When_InputOk() {
		//given
		BookingRequest bookingRequest = new BookingRequest("1", LocalDate.of(2021,7,1),
				LocalDate.of(2021,7,5),4, true);

		//when
		String bookingId = bookingService.makeBooking(bookingRequest);

		//then
		verify(bookingDAOMock).save(bookingRequest);
		System.out.println(bookingId);
	}

	@Test
	void should_CancelBooking_When_InputOK() {
		//given
		BookingRequest bookingRequest = new BookingRequest("1", LocalDate.of(2021,7,1),
				LocalDate.of(2021,7,5),4, true);

		bookingRequest.setRoomId("1.3");
		String bookingId ="1";

		doReturn(bookingRequest).when(bookingDAOMock).get(bookingId);
		//When
		bookingService.cancelBooking(bookingId);

		//Then
		verify(bookingDAOMock).delete(bookingId);

	}



}
