package com.mockitotutorial.happyhotel.booking;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.*;

class Test10ArgumentCaptors {

	private BookingService bookingService;
	private PaymentService paymentServiceMock;
	private RoomService roomServiceMock;
	private BookingDAO bookingDAOMock;
	private MailSender mailSenderMock;
	private ArgumentCaptor<Double> doubleCaptor;
	//private ArgumentCaptor<BookingRequest> bookingRequestCaptor;


	@BeforeEach
	void setup() {
		paymentServiceMock = mock(PaymentService.class);
		roomServiceMock = mock(RoomService.class);
		bookingDAOMock = mock(BookingDAO.class);
		mailSenderMock = mock(MailSender.class);

		bookingService = new BookingService(paymentServiceMock, roomServiceMock, bookingDAOMock, mailSenderMock);

		this.doubleCaptor = ArgumentCaptor.forClass(Double.class);
	}

	@Test
	void should_PayCorrectPrice_When_InputOk() {
		//given
		BookingRequest bookingRequest = new BookingRequest("1", LocalDate.of(2021,7,1),
				LocalDate.of(2021,7,5),2, true);
		double expected = 400.0;
		//when
		bookingService.makeBooking(bookingRequest);

		//then
		verify(paymentServiceMock,times(1)).pay(any(),doubleCaptor.capture());
		double actual = doubleCaptor.getValue();

		assertEquals(expected, actual);
	}

	@Test
	void should_PayCorrectPrice_When_MultipleCalls() {
		//given
		BookingRequest bookingRequest = new BookingRequest("1", LocalDate.of(2021,7,1),
				LocalDate.of(2021,7,5),2, true);
		BookingRequest bookingRequest2 = new BookingRequest("1", LocalDate.of(2021,7,1),
				LocalDate.of(2021,7,5),4, true);

		List<Double>expected = Arrays.asList(400.0,800.0);

		//when
		bookingService.makeBooking(bookingRequest);
		bookingService.makeBooking(bookingRequest2);

		//then
		verify(paymentServiceMock,times(2)).pay(any(),doubleCaptor.capture());
		List<Double> actual = doubleCaptor.getAllValues();

		assertEquals(expected, actual);
	}

}
