package com.mockitotutorial.happyhotel.booking;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.*;
import org.mockito.junit.jupiter.MockitoExtension;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import static org.mockito.BDDMockito.*;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
class Test12MockitoBdd {

	@InjectMocks
	private BookingService bookingService;

	@Mock
	private PaymentService paymentServiceMock;

	@Mock
	private RoomService roomServiceMock;

	@Spy
	private BookingDAO bookingDAOMock;

	@Mock
	private MailSender mailSenderMock;

	@Captor
	private ArgumentCaptor<Double> doubleCaptor;

	@Test
	void should_CountAvailablePlaces_When_OneRoomAvailable() {
		//given
		given(roomServiceMock.getAvailableRooms())
				.willReturn(Collections.singletonList(new Room("Room2", 2)));
		int expected = 2;

		//when
		int actual = bookingService.getAvailablePlaceCount();

		//then
		assertEquals(expected,actual);
	}

	@Test
	void should_CalculateCorrectPrice_When_CorrectInput() {
		//given
		BookingRequest bookingRequest = new BookingRequest("1", LocalDate.of(2021,7,1),
				LocalDate.of(2021,7,5),4, true);
		//when
		bookingService.makeBooking(bookingRequest);

		//then
		then(paymentServiceMock).should(times(1)).pay(any(),eq(800.0));
		verifyNoMoreInteractions(paymentServiceMock);

	}


}
